<?php

namespace Drupal\module_builder\Form;

use Drupal\Core\Render\Element;
use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\module_builder\ExceptionHandler;
use Drupal\module_builder\LibraryWrapper;
use DrupalCodeBuilder\Exception\SanityException;


/**
 * Class ModuleBuilderModuleHooksForm
 *
 * Form for selecting hooks.
 */
class ModuleBuilderModuleHooksForm extends ModuleBuilderComponentFormBase {

   /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $module = $this->entity;

    $data = $module->get('data');

    LibraryWrapper::loadLibrary();

    // Get the Task handler.
    try {
      $mb_task_handler_report = \DrupalCodeBuilder\Factory::getTask('ReportHookData');
    }
    catch (SanityException $e) {
      ExceptionHandler::handleSanityException($e);
      return $form;
    }

    // Call a method in the Task handler to perform the operation.
    $hook_info = $mb_task_handler_report->listHookOptionsStructured();
    // Need to catch \DrupalCodeBuilder\Exception!

    // Create a fieldset for each group, containing checkboxes.
    foreach ($hook_info as $group => $hook_group_info) {
      $form[$group] = array(
        '#type' => 'details',
        '#title' => $group,
        //'#open' => TRUE,
      );

      $hook_names = array_keys($hook_group_info);

      // Need to differentiate the key, otherwise FormAPI treats this as an
      // error on submit.
      $group_default_value = isset($data['hooks']) ? array_intersect($hook_names, $data['hooks']) : [];
      $form[$group][$group . '_hooks'] = array(
        '#type' => 'checkboxes',
        '#options' => array_combine($hook_names, $hook_names),
        '#default_value' => $group_default_value,
      );

      if (!empty($group_default_value)) {
        $form[$group]['#open'] = TRUE;
      }

      foreach ($hook_group_info as $hook => $hook_info_single) {
        $description = $hook_info_single['description'];

        if ($hook_info_single['core']) {
          // External Uri.
          $url = Url::fromUri('https://api.drupal.org/api/function/' . $hook . '/8');
          $description .= ' ' . \Drupal::l(t('[documentation]'), $url);
        }

        $form[$group][$group . '_hooks'][$hook]['#description'] = $description;
      }
    }

    $form_state->set('module_builder_groups', array_keys($hook_info));

    return $form;
  }

  /**
   * Copies top-level form values to entity properties
   *
   * This should not change existing entity properties that are not being edited
   * by this form.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity the current form should operate upon.
   * @param array $form
   *   A nested array of form elements comprising the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  protected function copyFormValuesToEntity(EntityInterface $entity, array $form, FormStateInterface $form_state) {
    $values = $form_state->getValues();

    // We can't just iterate over $values, because of a core bug with
    // EntityForm, so we need to know which keys to look at.
    // See https://www.drupal.org/node/2665714.
    $groups = $form_state->get('module_builder_groups');

    $hooks = [];
    foreach ($groups as $group) {
      $group_values = $values[$group . '_hooks'];
      // Filter out empty values. (FormAPI *still* doesn't do this???)
      $group_hooks = array_filter($group_values);
      // Store as a numeric array.
      $group_hooks = array_keys($group_hooks);

      $hooks = array_merge($group_hooks, $hooks);;
    }

    $data = $entity->get('data');
    $data['hooks'] = $hooks;
    $entity->set('data', $data);
  }

}
